unit INITGestalt;

{	Based on a proposal by Jeremy Roussak (jeremyr@dcs.qmw.ac.uk) & Ren G.A. Ros (rgaros@bio.vu.nl) }
{	Pascal header file by Ren G.A. Ros (rgaros@bio.vu.nl) }
{	C header file also available, by Dave Ely (ely@netcom.com) }
{	This header file complies to version 1.0 of the INITGestalt proposal }
{	The functions included with this unit are NOT part of the INITGestalt proposal }

{	Changes (European date format!): }
{	12/01/95	RR	Created, 1.0a6 proposal }
{	15/01/95	RR	Modified for INITGestalt 1.0a7, see standard for changes}
{								corrected a few mistakes}
{								removed custAttrFlags parameter from MakeMinimalIGRec}
{	18/01/95	RR	Modified for INITGestalt 1.0a8, see standard for changes}
{								corrected assigning the PSN in the MakeIGRec function}
{								corrected ReplaceINITGestalt and DeleteINITGestalt by making oldIGRec a var parameter}
{	22/01/95	RR	Modified for INITGestalt 1.0a9, see standard for changes}
{	24/01/95	RR	Modified for INITGestalt 1.0a10, see standard for changes}
{	31/01/95	RR	Modified for INITGestalt 1.0b1, see standard for changes}
{	20/02/95	RR	Modified for INITGestalt 1.0b2, see standard for changes}
{	21/03/95	RR	Modified for INITGestalt 1.0, see standard for changes}

{NOT TESTED WITH MW CW YET! ONLY USED WITH THINK PASCAL 4.0.2}

interface

	uses
{$IFC THINK_Pascal}
		GestaltValue, Processes;
{$ELSEC}
		Errors, GestaltValue, MixedMode, Processes, Types;
{$ENDC}

{$IFC THINK_Pascal}
	type
		UniversalProcPtr = ProcPtr;
		SInt8 = SignedByte;
		ISAType = SInt8;

	const
		kM68kISA = 0;
		kPowerPCISA = 1;

		GetCurrentISA = kM68kISA;
{$ENDC}

	type
		INITGestaltRec = record
				recType: OSType;							{always kGestaltRecordType}
				recVersion: NumVersion;				{version of the record format}
				igVersion: NumVersion;					{version of installed software}
				igStdAttr: LongInt;						{standard defined attribute bits}
				reserved1: LongInt;						{reserved, more attribute bits}
				igCustomAttr: LongInt;					{custom defined attribute bits}
				igProcess: ProcessSerialNumber;	{PSN of installer}
				igPlatformType: ISAType;				{platform type}
				reserved2: Byte;							{reserved}
				reserved3: Integer;						{reserved}
				igRefCon: LongInt;							{private pointer or handle}
			end;

		INITGestaltRecPtr = ^INITGestaltRec;
		INITGestaltRecHandle = ^INITGestaltRecPtr;

	const
		kIGRecType = 'JRRR';
		kIGRecCurrentVersion = $01008000;

		{bit constants for the stdAttr field}
		kIGEnabled = 0;
		kIGDebugVersion = 1;
		kIGDebugEnabled = 2;
		kIGAcceptsAEvents = 3;


{ The functions listed below are not part of the INITGestalt proposal, }
{ but you're advised to use them because of future compatibility. }

	const
		gestaltINITGestaltTable = kIGRecType;

{ They provide a level of abstraction between your software and the way in which the INITGestalt }
{ record address is actually registered with the Gestalt Manager. }
{ Note that your software does not need to keep it's own copy of the address of the INITGestalt }
{ record. Your software can get the address by calling Gestalt also, and the functions are designed to }
{ help you with this. For example, DeleteINITGestalt returns the address which was registered with }
{ the Gestalt Manager which allows you to dispose of it. Allocating and disposing of the INITGestalt }
{ record is not part of  NewINITGestalt, ReplaceINITGestalt and DeleteINITGestalt because it may be }
{ part of a larger structure. }

{ EXAMPLE: }
{ const }
{	kCreator = '????'; }
{	kVersion = $06006005; }
{	kDefaultStdFlags = $00000001;							(only kIGEnabled bit set) }
{ }
{ var }
{	igrec: INITGestaltRecPtr; }
{	err: OSErr; }

{ When initializing your software: }
{ err := AllocateIGRec(igrec);									(or use AllocateSysIGRec) }
{ }
{ if (err = noErr) then }
{		begin }
{		err := MakeMinimalIGRec(igrec, NumVersion(kVersion), kDefaultStdFlags); }
{																				(or use MakeIGRec) }
{ }
{ 		if (err = noErr) then }
{				err := NewINITGestalt(kCreator, igrec); }
{		end; }

{ In between your software can change some of the fields of the INITGestaltRec. }
{ If you lost the pointer to your INITGestaltRec you can use Gestalt to get it back. This example }
{ has the pointer stored in a local variable and would need to call Gestalt. }

{ To clean up when quitting: }
{ err := ReplaceINITGestalt(kCreator, 0, igrec); }
{				( Note that ReplaceINITGestalt does not need the pointer as used with NewINITGestalt). }
{				( Also note that DeletINITGestalt, which uses DeleteGestaltValue, deletes the whole) }
{				( Gestalt selector which would not allow to have it return zero. Thus ReplaceINITGestalt) }
{				( is used here.) }
{ }
{ if (err = noErr) then }
{		err := DisposeIGRec(igrec); }


{--------------------------------------------------------------------------------}
{ AllocateSysIGRec }
{ Function to allocate a non-relocatable memory block in the System Heap }
{ Your software should use DisposeIGRec to dispose the allocated memory block again, }
{ for compatibilty. }
{ input:		none }
{ output:	IGRec:					address of a newly allocated INITGestaltRec in the System Heap }
{				result:					error code (noErr if no error occured) }
{--------------------------------------------------------------------------------}
	function AllocateSysIGRec (var IGRec: INITGestaltRecPtr): OSErr;

{--------------------------------------------------------------------------------}
{ DisposeIGRec }
{ Function to dispose of the non-relocatable INITGestaltRec memory block }
{ input:		IGRec:					address of a allocated (by AllocateIGRec or AllocateSysIGRec) }
{											INITGestaltRec record }
{ output:	IGRec:					nil value if dispose was sucessfull }
{				result:					error code (noErr if no error occured) }
{--------------------------------------------------------------------------------}
	function DisposeIGRec (var IGRec: INITGestaltRecPtr): OSErr;

{--------------------------------------------------------------------------------}
{ NewINITGestalt }
{ Function to install the address of the INITGestaltRec as the response for the specified }
{ Gestalt selector. }
{ input:		selector:				Gestalt Selector (equal to signature) }
{    			newIGRec: 			address of a allocated INITGestaltRec record }
{ output:	result:					error code (noErr if no error occured) }
{--------------------------------------------------------------------------------}
	function NewINITGestalt (selector: OSType; newIGRec: INITGestaltRecPtr): OSErr;

{--------------------------------------------------------------------------------}
{ ReplaceINITGestalt }
{ Function to install the address of a INITGestaltRec as the response for the specified }
{ Gestalt selector, while returning the address of a previously installed INITGestaltRec }
{ to be disposed of by the calling software. }
{ input:		selector:				Gestalt Selector (equal to signature) }
{    			newIGRec: 			address of the new allocated INITGestaltRec record }
{    			oldIGRec: 				address of earlier installed INITGestaltRec record }
{ output:	result:					error code (noErr is no error occured) }
{--------------------------------------------------------------------------------}
	function ReplaceINITGestalt (selector: OSType; newIGRec: INITGestaltRecPtr; var oldIGRec: INITGestaltRecPtr): OSErr;

{--------------------------------------------------------------------------------}
{ DeleteINITGestalt}
{ Function to delete the specified Gestalt selector, while returning the address of a }
{ previously installed INITGestaltRec to be disposed of by the calling software. }
{ input:		selector:				Gestalt Selector (equal to signature) }
{    			newIGRec: 			address of the new allocated INITGestaltRec record }
{    			oldIGRec: 				address of earlier installed INITGestaltRec record }
{ output:	result:					error code (noErr if no error occured) }
{--------------------------------------------------------------------------------}
	function DeleteINITGestalt (selector: OSType; var oldIGRec: INITGestaltRecPtr): OSErr;

{--------------------------------------------------------------------------------}
{ MakeMinimalIGRec}
{ Procedure to construct a INITGestaltRec with the minimal values for sofware not}
{ running as a process. }
{ input:		IGRec:					address of an allocated INITGestaltRec structure }
{    			version:				version of the software installing the INITGestaltRec }
{    			stdAttrFlags:		value for the stdAttr field of the INITGestaltRec record }
{											(see INITGestalt standard) }
{ output:	none }
{--------------------------------------------------------------------------------}
	procedure MakeMinimalIGRec (IGRec: INITGestaltRecPtr; version: NumVersion; stdAttrFlags: LongInt);

{--------------------------------------------------------------------------------}
{ MakeIGRec}
{ Procedure to construct a INITGestaltRec }
{ input:		IGRec:					address of an allocated INITGestaltRec structure }
{    			version:				version of the software installing the INITGestaltRec }
{				PSN:						pointer to process serial number of the software, or nil}
{    			stdAttrFlags:		value for the stdAttr field of the INITGestaltRec record }
{											(see INITGestalt standard) }
{    			custAttrFlags:		value for the custAttr field of the INITGestaltRec record }
{											(see INITGestalt standard) }
{				refCon:					for application's use}
{											(see INITGestalt standard) }
{ output:	none }
{--------------------------------------------------------------------------------}
	procedure MakeIGRec (IGRec: INITGestaltRecPtr; version: NumVersion; PSN: ProcessSerialNumberPtr; stdAttrFlags, custAttrFlags, refcon: LongInt);


implementation


	function AllocateSysIGRec (var IGRec: INITGestaltRecPtr): OSErr;
	begin
		IGRec := INITGestaltRecPtr(NewPtrSys(SizeOf(INITGestaltRec)));
		AllocateSysIGRec := MemError;
	end;


	function DisposeIGRec (var IGRec: INITGestaltRecPtr): OSErr;
		var
			err: OSErr;
	begin
		DisposePtr(Ptr(IGRec));
		err := MemError;

		if (err = noErr) then
			IGRec := nil;

		DisposeIGRec := err;
	end;


	function AddToIGSelectorsList (selector: OSType): OSErr;
{ function to add newly installed selector to a separate list of selectors which support the INITGestalt }
{ proposal. The Gestalt selector gestaltINITGestaltTable returns a handle to this list, which is in a }
{ relocatable block in the system heap and is a plain list of OSType. }
		type
			igSelectorType = OSType;
			igSelectorsTable = array[0..0] of igSelectorType;
															{ list of selectors }
			igSelectorsTablePtr = ^igSelectorsTable;
			igSelectorsTableH = ^igSelectorsTablePtr;
		var
			igTableAddr: igSelectorsTableH;		{ Pointer to the list of selectors }
			currentElement: Byte;						{ index of the current element in the igSelectorType list }
			elementExists: Boolean;					{ TRUE if selector already has an entry in the list }
			err: OSErr;										{ error message }
	begin
		err := Gestalt(gestaltINITGestaltTable, LongInt(igTableAddr));
															{ get handle to the igSelectorType list }


		if (err = gestaltUndefSelectorErr) then
															{ no lists exists yet }
			begin
				igTableAddr := igSelectorsTableH(NewHandleSys(SizeOf(igSelectorsTable)));
				err := MemError;								{ create new list with the dummy element }

				if (igTableAddr <> nil) then				{ got a block to work with }
					begin
						igTableAddr^^[0] := igSelectorType(MAXINT);
															{ last entry in list is a dummy with value MAXINT }

						err := NewGestaltValue(gestaltINITGestaltTable, LongInt(igTableAddr));
															{ register selector with handle}
					end;

			end;


		if (err = noErr) & (igTableAddr <> nil) then
															{ the list was either already installed or we just installed it: }
															{ all's well and there is a list. }
			begin

{ search thru the existing list if the selector already exists. We don't need to do this since this code is only }
{ called the first time a selector is installed. The error message returned by NewGestaltValue makes sure}
{ this code is skipped. But for safety, and we need to find the last element anyway. }

				currentElement := 0;							{ start with the first element }
				elementExists := FALSE;					{ nothing found yet }
				repeat
					elementExists := (igTableAddr^^[currentElement] = selector);
															{ is the selector in this element? }
					currentElement := currentElement + Byte(elementExists);
															{ if the selector wasn't in this element, go to the next one }
				until ((igTableAddr^^[currentElement] = igSelectorType(MAXINT)) | elementExists);
															{ repeat until we're at the end of the list, or found the selector }

{ if selectors was in list nothing needs to be done. All is well, so don't return an error. }

				if not elementExists then				{ if the selector isn't in the list }
					begin
						SetHandleSize(Handle(igTableAddr), GetHandleSize(Handle(igTableAddr)) + SizeOf(igSelectorType));
						err := MemError;							{ increase the list for one more element }

						if (err = noErr) then						{ increasing block size went fine }
							begin
								igTableAddr^^[currentElement] := selector;
															{ the current (previously last) element now holds the newly added selector }
								igTableAddr^^[currentElement + 1] := igSelectorType(MAXINT);
															{ the last element gets the dummy again }
							end;

					end;

			end
		else
			err := nilHandleErr;

		AddToIGSelectorsList := err;				{ readu, return error message }
	end;


	function NewINITGestalt (selector: OSType; newIGRec: INITGestaltRecPtr): OSErr;
		var
			err: OSErr;
	begin
		err := NewGestaltValue(selector, LongInt(newIGRec));

		if (err = noErr) then
			err := AddToIGSelectorsList(selector);

		NewINITGestalt := err;
	end;


	function ReplaceINITGestalt (selector: OSType; newIGRec: INITGestaltRecPtr; var oldIGRec: INITGestaltRecPtr): OSErr;
		var
			err: OSErr;
	begin
		err := Gestalt(selector, LongInt(oldIGRec));

		if (err = noErr) then
			err := ReplaceGestaltValue(selector, LongInt(newIGRec));

		ReplaceINITGestalt := err;
	end;


	function DeleteINITGestalt (selector: OSType; var oldIGRec: INITGestaltRecPtr): OSErr;
		var
			err: OSErr;
	begin
		err := Gestalt(selector, LongInt(oldIGRec));

		if (err = noErr) then
			err := DeleteGestaltValue(selector);

		DeleteINITGestalt := err;
	end;


	procedure MakeMinimalIGRec (IGRec: INITGestaltRecPtr; version: NumVersion; stdAttrFlags: LongInt);
	begin
		MakeIGRec(IGRec, version, nil, stdAttrFlags, 0, 0)
	end;


	procedure MakeIGRec (IGRec: INITGestaltRecPtr; version: NumVersion; PSN: ProcessSerialNumberPtr; stdAttrFlags, custAttrFlags, refcon: LongInt);
		var
			err: OSErr;
	begin

		IGRec^.recType := kIGRecType;
		IGRec^.recVersion.version := kIGRecCurrentVersion;
		IGRec^.igVersion := version;
		IGRec^.igStdAttr := stdAttrFlags;
		IGRec^.reserved1 := 0;
		IGRec^.igCustomAttr := custAttrFlags;

		if (PSN = nil) then
			begin
				IGRec^.igProcess.highLongOfPSN := 0;
				IGRec^.igProcess.lowLongOfPSN := kNoProcess;
			end
		else
			IGRec^.igProcess := PSN^;

		IGRec^.igPlatformType := GetCurrentISA;
		IGRec^.reserved2 := 0;
		IGRec^.reserved3 := 0;
		IGRec^.igRefCon := refcon;

	end;

end.